<?  
  /**
    * VarManager - forward and track vars with ease.
    *
    * Class for holding current state of web variables and their
    * respective forwarding status.  Maintains state as well as
    * generation of input forms and URIs.<br><br>
    * For the latest release and information visit
    * {@link http://dave.thehorners.com http://dave.thehorners.com}
    *
    * @copyright David Horner 2004-2005
    * @license http://www.gnu.org/copyleft/lesser.html LGPL License
    * @version 1.04
    * @author David A. Horner
    * {@link http://dave.thehorners.com Homepage}
    * @tutorial ./example.php
    * @package VarManager
    */
    class VarManager
    {
        /** list of forwarded vars. */
        var $forwardlist=null;   
        /** list of overridden vars. */
        var $overrides=null;     
        /** separator used when generating URIs. */
        var $separator="&amp;";  
        /** relative name of script to refer to in URIs. */
        var $relname;           
        /** absolute url of script to refer to in URIs. */
        var $url;                

        /**
          * Creates a VarManager object that doesn't forward/override anything
          * Reads default values from environment.
          * @return object.
          */
        function VarManager()
        {
            $this->reset();
        }

        /**
          * Resets object to its initial default state.
          * @return void.
          */
        function reset()
        {
            $this->relname=$_SERVER['PHP_SELF'];
            $this->url=$this->generate_baseurl();
            $separator="&amp;";
            $this->forwardlist=array();
            $this->overrides=array();
        }
        
        /**
          * Returns a copy of the VarManager object for temporary use.
          * @return object.
          */
        function makedup()
        {
            $ret=new varmanager();
            $ret->forwardlist=$this->forwardlist;
            $ret->overrides=$this->overrides;
            $ret->separator=$this->separator;
            $ret->relname=$this->relname;
            $ret->url=$this->url;
            return $ret;
        }
        
        /**
          * Sets string representing separator for URI generation.
          * @param string $separator string of separator
          * @return void.
          */
        function set_separator($separator)
        {
            $this->separator=$separator;
        }
        
        /**
          * Sets string representing current url for absolute URI generation.
          * @param string $server_url string of absolute url
          * @return void.
          */
        function set_url($server_url)
        {
            if(strlen($server_url)==0) {
                $this->url=generate_baseurl();
            } else {
                $this->url=$server_url;
            }
        }
        
        /**
          * Returns string representing current url for absolute URI generation.
          * @return string.
          */
        function get_url()
        {
            return $this->url;
        }

        /**
          * Returns string representing relative name for relative URI generation.
          * @return string.
          */
        function get_relname()
        {
            return $this->relname;
        }

        /**
          * Sets string representing relative name for relative URI generation.
          * @param string $relname string of relative name
          * @return void.
          */
        function set_relname($relname)
        {
            $this->relname=$relname;
        }
    
        /**
          * Returns string representing absolute url to current script.
          * @return string.
          */
        function generate_baseurl()
        {
            global $HTTP_HOST,$SCRIPT_NAME,$HTTPS,$SERVER_PORT;
          
            $ret="http";
            if($HTTPS=="on")
                $ret.="s";
            $ret.="://".$HTTP_HOST.":".$SERVER_PORT;
            $ret.=$SCRIPT_NAME;
            return $ret;
        }

        /**
          * Returns a string with uri using requested overrides and forwarded variables.
          * @param boolean $isabsolute if true an absolute URI is produced.
          * @return string.
          */
        function generate_uri($isabsolute=false)
        {
            $questionsent=false;
            $ret="";
        
            if ($isabsolute==true) {
                $ret=$this->get_url();
            } else {
                $ret=$this->get_relname();
            }
        
            $ret.=$this->__loopvars("__printurivalue");
            
            return $ret;
        }        
        
        /**
          * Replaces {uri} in content with generated uri using requested overrides/forwards.
          * @param string $content text to be searched and replaced.
          * @param boolean $isabsolute if true an absolute URI is produced.
          * @return string.
          */
        function expand_string($content,$isabsolute=false)
        {
          $patterns = array("#{URI(.*?)}#se",
                            "#(VAR(.*?))#se"
                            );
          $replacements = array(
                              "\$this->__expandstring('\\1','$isabsolute')",
                              "\$this->var_val('\\1');"
                              );
          return preg_replace($patterns,$replacements,$content);
        }

        function __expandstring($contents,$isabsolute)
        {
          $contents=trim($contents);
          if(strlen($contents)==0) {
            return $this->generate_uri($isabsolute);
          } else {
            return $this->__expandstringargs($contents,$isabsolute);
          }
        }

        function __expandstringargs($contents,$isabsolute)
        {
          $contents=trim($contents);
          echo "CONTENTS (".$contents.")";
          $tmp=$this->makedup();
          $patterns = array(
                              "/reset()/e",
                              "/forwardall()/e",
                              "/unset()/e"
                            );
          $replacements = array(
                              "\$tmp->reset();",
                              "\$tmp->forwardall();",
                              "\$tmp->var_unset($1);"
                              );
          preg_replace($patterns,$replacements,$contents);
          return $tmp->generate_uri($isabsolute);
        }

        /**
          * Replaces {uri} in content with generated uri using requested overrides/forwards.
          * @param string $content text to be searched and replaced.
          * @param boolean $isabsolute if true an absolute URI is produced.
          * @return string.
          */
        function replace_uri($content,$isabsolute=false, $string_to_replace="{URI}")
        {
          $link=$this->generate_uri($isabsolute);
          return str_replace($string_to_replace, $link, $content);
        }
        
        /**
          * Prints uri to stdout.
          * @param boolean $isabsolute if true an absolute URI is produced.
          * @return void.
          */
        function print_uri($isabsolute=false)
        {
            echo $this->generate_uri($isabsolute);
        }
        
        /**
          * prints a form tag with action set to current relname/absolute uri.
          * @param boolean $isabsolute if true an absolute URI is produced.
          * @param string $extraattr additional attributes to add to form tag.
          * @return void.
          */
        function print_form($isabsolute=false,$extraattr="")
        {
            if ($isabsolute==true) {
                $ret=$this->get_url();
            } else {
                $ret=$this->get_relname();
            }
            echo "\n<form action=\"$ret\"$extraattr>\n";
        }

        /**
          * Generates string containing hidden input tags.
          * @return string.
          */
        function generate_hidden() 
        {
            $ret=$this->__loopvars("__printhiddenvalue");
            return $ret;
        }
                
        /**
          * Prints hidden input tags to stdout.
          * @return void.
          */
        function print_hidden()
        {
            echo $this->generate_hidden();
        }

        /**
          * Prints information about the current object to stdout.
          * @return void.
          */
        function dumpinfo()
        {
            echo "\n<pre><u><b>----DUMP INFO---</b></u>\n";
            echo "<b>URL:=</b>" . $this->get_url() . "\n";
            echo "<b>RELNAME:=</b>" . $this->get_relname() . "\n";
            echo "<b>SEPARATOR:=</b>" . htmlentities($this->separator) . "\n";
            echo "<b>CURRENT REL. URI:=</b>" . $this->generate_uri(false) . "\n";
            echo "<b>CURRENT ABS. URI:=</b>" . $this->generate_uri(true) . "\n";
            echo "\n<b>REQUEST ARRAY</B>\n";
            var_dump($_REQUEST);
            echo "\n<b>FORWARD ARRAY</B>\n";
            var_dump($this->forwardlist);
            echo "\n<b>OVERRIDES ARRAY</B>\n";
            var_dump($this->overrides);
            echo "<b>----DUMP INFO---</b></pre>\n";
        }

        /**
          * Forwards all the variables in the current environment.
          * @return void.
          */
        function forwardall()
        {
            foreach($_REQUEST as $name => $val) {
                $this->forwardlist[$name]=true;
            }
        }

        /**
          * Forwards all the variables in the current environment beginning with given string.
          * @param string $begins_with string to compare environment with.
          * @param boolean $ignore_case controls matching of case during compare.
          * @return void.
          */
        function forward_beginswith($begins_with,$ignore_case=false)
        {
            $len=strlen($begins_with);
            if($ignore_case) {
                foreach($_REQUEST as $name => $val) {
                    if(strncasecmp($begins_with,$name,$len)==0) {
                        $this->forwardlist[$name]=true;
                    }
                }
            } else {
                foreach($_REQUEST as $name => $val) {
                    if(strncmp($begins_with,$name,$len)==0) {
                        $this->forwardlist[$name]=true;
                    }
                }
            }
        }
        
        /**
          * Forwards all the variables in the current environment beginning with given string.
          * @param string $begins_with string to compare environment with.
          * @param boolean $ignore_case controls matching of case during compare.
          * @return void.
          */
        function forward_beginswith_except($begins_with,$exceptvars,$ignore_case=false)
        {
            $this->forward_beginswith($begins_with,$ignore_case);
            $this->var_unforward($exceptvars);
        }        
        
        /**
          * Forward all the variables except for those named.
          * @param mixed string or array of the variables to not forward.
          * @return void.
          */
        function forwardall_except($exceptvars)
        {
            $this->forwardall();
            $this->var_unforward($exceptvars);            
        }

        /**
          * Forward the value of the variables as they exist in current environment.
          * @param array name=>value pairs of the variables to forward.
          * @return void.
          */
        function forward($forwardvars)
        {
            if(is_array($forwardvars)) {
                foreach($forwardvars as $val => $name) {
                    $this->forwardlist[$name]=true;
                }
            } else if(is_string($forwardvars)) {
                $this->forwardlist[$forwardvars]=true;
            }
        }

        /**
          * Overrides the value of the variables as they exist in current environment.
          * @param array name=>value pairs of the variables to override.
          * @return void.
          */
        function override($overridevars)
        {
            if(is_array($overridevars)) {
                foreach($overridevars as $name => $val) {
                    $this->overrides[$name]=$val;
                }
            } else if(is_string($overridevars)) {
                $this->overrides[$overridevars]=true;
            }
        }

        function var_val($name)
        {
            if(isset($this->overrides[$name])) {
              return $this->overrides[$name];
            }
            return $_REQUEST[$name];
        }

        /**
          * Returns the value of the variable as it exists in current environment.
          * @param string $name name of the variable to pull value from env.
          * @return mixed.
          */
        function var_sent($name)
        {
            return $_REQUEST[$name];
        }
                         
        /**
          * Returns true if the variable exists in current environment or overrides.
          * @param string $name name of the variable to test for existence.
          * @return boolean.
          */
        function var_exists($name)
        {
          if($this->var_issent($name) || $this->var_isoverride($name))
            return true;
          return false;
        }


        function var_issent($name)
        {
            return isset($_REQUEST[$name]);
        }

        /**
          * Returns boolean indicating if given variable is forwarded.
          * @param string $name name of the variable to check status on.
          * @return boolean.
          */
        function var_isforward($name)
        {
          return isset($this->forwardlist[$name]);
        }

        /**
          * Returns boolean indicating if given variable is overridden.
          * @param string $name name of the variable to check status on.
          * @return boolean.
          */
        function var_isoverride($name)
        {
          return isset($this->overrides[$name]);
        }

        /**
          * Removes all forwards for given variable.
          * @param string $name name of the variable to unforward.
          * @return boolean.
          */
        function var_unforward($name)
        {
            if(is_string($name)) {
                if($this->var_isforward($name)) {
                    unset($this->forwardlist[$name]);
                    return true;
                }
            } else if(is_array($name)) {
                $ret=false;
                foreach($name as $tval => $tname) {
                    $ret=$this->var_unforward($tname);
                }
                return $ret;
            }
            return false;
        }
        
        /**
          * Removes all overrides for given variable.
          * @param string $name name of the variable to unoverride.
          * @return boolean.
          */
        function var_unoverride($name)
        {
            if(is_string($name)) {
                if($this->var_isoverride($name)) {
                    unset($this->overrides[$name]);
                    return true;
                }
            } else if(is_array($name)) {
                $ret=false;
                foreach($name as $tval => $tname) {
                    $ret=$this->var_unoverride($tname);
                }
                return $ret;
            }
            return false;
        }

        /**
          * Removes all forwards and overrides for given variable.
          * @param string $name name of the variable to unset.
          * @return void.
          */
        function var_unset($name)
        {
          $this->var_unoverride($name);
          $this->var_unforward($name);
        }
        
        /** __printurivalue (private method)
          * @internal
          */
        function __printurivalue($name,$val,$numsent,$isarray)
        {
            if($numsent<=0) {
                $ret="?";  
            } else {
                $ret=$this->separator;
            }
            
            $ret.=urlencode($name);
            
            if($isarray==true)
                $ret.="[]";
            
            if(strlen($val)>0)
                $ret.= "=" . urlencode($val);
            return $ret;
        }

        /** __printhiddenvalue (private method)
          * @internal
          */
        function __printhiddenvalue($name,$val,$numsent,$isarray)
        {
            $ret="<input type=\"hidden\" name=\"".urlencode($name);
            if($isarray==true)
                $ret.="[]";            
            $ret.="\"";
            if(strlen($val))
                $ret.= " value=\"".urlencode($val)."\"";
            $ret.=">\n";
            return $ret;
        }

        /** __callbackvar (private method)
          * @internal
          */
        function __callbackvar($render_callback,$name,$val,$numsent)
        {
          $ret="";
          if(is_array($val)) {
              foreach($val as $sname => $sval){
                  $ret.=$this->$render_callback($name,$sval,$numsent,true); 
              }
          } else {
              $ret.=$this->$render_callback($name,$val,$numsent,false); 
          }
          return $ret;
        }

        /** __loopvars (private method)
          * @internal
          */
        function __loopvars($render_callback)
        {
            $ret="";
            $numsent=0;
            // Forward on existing vars and overwritten vars that exist in env.
            foreach($this->forwardlist as $name => $fval){
                if(isset($_REQUEST[$name]) || isset($this->overrides[$name])) {
                    if(isset($this->overrides[$name])) {
                        $val=$this->overrides[$name];
                    } else {
                        $val=$_REQUEST[$name];
                    }
                    $ret.=$this->__callbackvar($render_callback,$name,$val,$numsent++);
                }
            }
            // Send overrides that didn't already exist in env.
            foreach($this->overrides as $name => $val) {
                // overridden but not forwarded
                if(!isset($this->forwardlist[$name])) {
                  $ret.=$this->__callbackvar($render_callback,$name,$val,$numsent++);               
                }
            }
            return $ret;
        }
  } 
?>